#!/usr/bin/env python


from importlib.resources import open_binary
import json
from random import Random
import unittest


from . import test as bioarch_test
from .left_right import LeftRight
from .occupational_markers import EnthesialMarker, OccupationalMarkers


class EnthesialMarkerTest(unittest.TestCase):
    def test_parse(self):
        self.assertEqual(EnthesialMarker.parse(None), None)

        self.assertEqual(EnthesialMarker.parse(0), EnthesialMarker(0))
        self.assertEqual(EnthesialMarker.parse(0.0), EnthesialMarker(0))

        self.assertEqual(EnthesialMarker.parse(0.0), EnthesialMarker(0.0, is_s=False, is_oe=False))
        self.assertEqual(EnthesialMarker.parse(0.5), EnthesialMarker(0.5, is_s=False, is_oe=False))

        self.assertEqual(EnthesialMarker.parse(3.5), EnthesialMarker(0.5, is_s=True, is_oe=False))
        self.assertEqual(EnthesialMarker.parse(4.0), EnthesialMarker(1.0, is_s=True, is_oe=False))

        self.assertEqual(EnthesialMarker.parse(6.5), EnthesialMarker(0.5, is_s=False, is_oe=True))
        self.assertEqual(EnthesialMarker.parse(7.0), EnthesialMarker(1.0, is_s=False, is_oe=True))

        self.assertEqual(EnthesialMarker.parse('.5'), EnthesialMarker(0.5))
        self.assertEqual(EnthesialMarker.parse('r.5'), EnthesialMarker(0.5))
        self.assertEqual(EnthesialMarker.parse('S.5'), EnthesialMarker(0.5, is_s=True))
        self.assertEqual(EnthesialMarker.parse('OE3'), EnthesialMarker(3, is_oe=True))

        with self.assertRaises(ValueError):
            EnthesialMarker.parse('')

    def test_construction(self):
        EnthesialMarker(0)
        with self.assertRaises(ValueError):
            EnthesialMarker(-1)
        with self.assertRaises(ValueError):
            EnthesialMarker(4)
        with self.assertRaises(ValueError):
            EnthesialMarker(0.1)
        with self.assertRaises(ValueError):
            EnthesialMarker(0.5, is_s=True, is_oe=True)
        with self.assertRaises(ValueError):
            EnthesialMarker(0, is_s=True)
        with self.assertRaises(ValueError):
            EnthesialMarker(0, is_oe=True)

    def test_equality(self):
        self.assertEqual(EnthesialMarker.parse('S.5'), EnthesialMarker.parse('S.5'))

        self.assertNotEqual(EnthesialMarker.parse('S.5'), EnthesialMarker.parse('.5'))
        self.assertNotEqual(EnthesialMarker.parse('S.5'), None)
        self.assertNotEqual(None, EnthesialMarker.parse('S.5'))

        with self.assertRaises(NotImplementedError):
            EnthesialMarker.parse('S.5').__eq__('hello')

    def test_avg(self):
        va1 = EnthesialMarker.parse('S.5')
        va2 = EnthesialMarker.parse('S.5')
        self.assertEqual(EnthesialMarker.avg(va1, va2), EnthesialMarker.parse('S.5'))

    def test_as_num(self):
        self.assertEqual(EnthesialMarker.parse('0').as_num(), 0.0)
        self.assertEqual(EnthesialMarker.parse('.5').as_num(), 0.5)
        self.assertEqual(EnthesialMarker.parse('3').as_num(), 3.0)
        self.assertEqual(EnthesialMarker.parse('s.5').as_num(), 3.5)
        self.assertEqual(EnthesialMarker.parse('s3').as_num(), 6.0)
        self.assertEqual(EnthesialMarker.parse('oe0.5').as_num(), 6.5)
        self.assertEqual(EnthesialMarker.parse('oe3').as_num(), 9.0)


class OccupationalMarkersTest(unittest.TestCase):
    def setUp(self):
        self.random = Random(666)

    def random_em(self):
        return EnthesialMarker.parse(self.random.choice((None, 0, 1.5, 3.0, 3.5, 6.5)))

    def test_to_pd_data_frame_values(self):
        df = OccupationalMarkers(*[LeftRight(self.random_em(), self.random_em()) for _ in range(0, 67)]).to_pd_data_frame('id1')

        with open_binary(bioarch_test, 'OccupationalMarkersTest.test_to_pd_data_frame_values.json') as json_stream:
            expected_json = json.load(json_stream)

        print(df.to_json(orient='records'))
        actual_json = json.loads(df.to_json(orient='records'))

        self.assertEqual(actual_json, expected_json)

    def test_to_pd_data_frame_empty(self):
        df = OccupationalMarkers.empty().to_pd_data_frame('id1')

        with open_binary(bioarch_test, 'OccupationalMarkersTest.test_to_pd_data_frame_empty.json') as json_stream:
            expected_json = json.load(json_stream)

        print(df.to_json(orient='records'))
        actual_json = json.loads(df.to_json(orient='records'))

        self.assertEqual(actual_json, expected_json)


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


import unittest


from .left_right import LeftRight
from .trauma import Trauma, TraumaCategory


class TraumaCategoryTest(unittest.TestCase):
    def test_parse(self):
        self.assertEqual(TraumaCategory.parse('NA'), TraumaCategory.NOT_PRESENT)
        self.assertEqual(TraumaCategory.parse('-1'), TraumaCategory.NOT_PRESENT)
        self.assertEqual(TraumaCategory.parse('0.5'), TraumaCategory.PARTIAL_BONE)
        self.assertEqual(TraumaCategory.parse('1'), TraumaCategory.NORMAL)
        self.assertEqual(TraumaCategory.parse('2'), TraumaCategory.INFECTION)
        self.assertEqual(TraumaCategory.parse('3'), TraumaCategory.FRACTURE)
        self.assertEqual(TraumaCategory.parse('4'), TraumaCategory.UNHEALED_FRACTURE)
        self.assertEqual(TraumaCategory.parse('5'), TraumaCategory.CRIBA)
        self.assertEqual(TraumaCategory.parse('6'), TraumaCategory.BLUNT_FORCE_TRAUMA)
        self.assertEqual(TraumaCategory.parse('7'), TraumaCategory.SHARP_FORCE_TRAUMA)
        self.assertEqual(TraumaCategory.parse('8'), TraumaCategory.TREPONATION)
        self.assertEqual(TraumaCategory.parse('9'), TraumaCategory.UNFUSED)
        self.assertEqual(TraumaCategory.parse('10'), TraumaCategory.BONY_GROWTH)
        self.assertEqual(TraumaCategory.parse('11'), TraumaCategory.FUSED)
        self.assertEqual(TraumaCategory.parse('12'), TraumaCategory.OSTEOCHONDRITIS_DESSICANS)

        self.assertEqual(TraumaCategory.parse('na'), TraumaCategory.parse('NA'))
        self.assertEqual(TraumaCategory.parse('FRACTURE'), TraumaCategory.parse('fracture'))
        self.assertEqual(TraumaCategory.parse('FRACTURE'), TraumaCategory.FRACTURE)
        self.assertEqual(TraumaCategory.parse(None), None)
        self.assertEqual(TraumaCategory.parse(TraumaCategory.FRACTURE), TraumaCategory.FRACTURE)

    def test_avg(self):
        self.assertEqual(TraumaCategory.avg(TraumaCategory.NOT_PRESENT, TraumaCategory.NOT_PRESENT), TraumaCategory.NOT_PRESENT)

        self.assertEqual(TraumaCategory.avg(TraumaCategory.NOT_PRESENT, TraumaCategory.PARTIAL_BONE), TraumaCategory.PARTIAL_BONE)
        self.assertEqual(TraumaCategory.avg(TraumaCategory.PARTIAL_BONE, TraumaCategory.NOT_PRESENT), TraumaCategory.PARTIAL_BONE)

        self.assertEqual(TraumaCategory.avg(TraumaCategory.PARTIAL_BONE, TraumaCategory.PARTIAL_BONE), TraumaCategory.PARTIAL_BONE)

        with self.assertRaises(NotImplementedError):
            TraumaCategory.avg(TraumaCategory.PARTIAL_BONE, TraumaCategory.NORMAL)


class TraumaTest(unittest.TestCase):
    def test_to_pd_data_frame(self):
        trauma = Trauma.empty()
        trauma.facial_bones = TraumaCategory.INFECTION
        trauma.clavicle = LeftRight(TraumaCategory.NOT_PRESENT, TraumaCategory.NORMAL)
        trauma.scapula = LeftRight(TraumaCategory.NORMAL, TraumaCategory.NOT_PRESENT)
        trauma.humerus = LeftRight(TraumaCategory.NORMAL, TraumaCategory.NORMAL)
        trauma.ulna = LeftRight(TraumaCategory.NORMAL, TraumaCategory.INFECTION)

        df = trauma.to_pd_data_frame('id1')

        self.assertEqual(df.to_json(orient='records'), '[{"clavicle_left_cat":"NOT_PRESENT","clavicle_left_val":-1,"clavicle_right_cat":"NORMAL","clavicle_right_val":1,"clavicle_avg_cat":"NORMAL","clavicle_avg_val":1,"scapula_left_cat":"NORMAL","scapula_left_val":1,"scapula_right_cat":"NOT_PRESENT","scapula_right_val":-1,"scapula_avg_cat":"NORMAL","scapula_avg_val":1,"humerus_left_cat":"NORMAL","humerus_left_val":1,"humerus_right_cat":"NORMAL","humerus_right_val":1,"humerus_avg_cat":"NORMAL","humerus_avg_val":1,"ulna_left_cat":"NORMAL","ulna_left_val":1,"ulna_right_cat":"INFECTION","ulna_right_val":2,"radius_left_cat":"NOT_PRESENT","radius_left_val":-1,"radius_right_cat":"NOT_PRESENT","radius_right_val":-1,"radius_avg_cat":"NOT_PRESENT","radius_avg_val":-1,"femur_left_cat":"NOT_PRESENT","femur_left_val":-1,"femur_right_cat":"NOT_PRESENT","femur_right_val":-1,"femur_avg_cat":"NOT_PRESENT","femur_avg_val":-1,"tibia_left_cat":"NOT_PRESENT","tibia_left_val":-1,"tibia_right_cat":"NOT_PRESENT","tibia_right_val":-1,"tibia_avg_cat":"NOT_PRESENT","tibia_avg_val":-1,"fibula_left_cat":"NOT_PRESENT","fibula_left_val":-1,"fibula_right_cat":"NOT_PRESENT","fibula_right_val":-1,"fibula_avg_cat":"NOT_PRESENT","fibula_avg_val":-1,"facial_bones_cat":"INFECTION","facial_bones_val":2,"ribs_cat":"NOT_PRESENT","ribs_val":-1,"vertabrae_cat":"NOT_PRESENT","vertabrae_val":-1}]')


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


from importlib.resources import open_binary
import json
import unittest


from . import test as bioarch_test
from .joints import JointCondition, Joints
from .left_right import LeftRight


class JointConditionTest(unittest.TestCase):
    def test_parse(self):
        self.assertEqual(JointCondition.parse(None), None)
        self.assertEqual(JointCondition.parse('NA'), None)
        self.assertEqual(JointCondition.parse('N'), None)
        self.assertEqual(JointCondition.parse('1'), JointCondition.MILD)
        self.assertEqual(JointCondition.parse('MILD'), JointCondition.MILD)

        self.assertEqual(JointCondition.parse('0'), JointCondition.NORMAL)
        self.assertEqual(JointCondition.parse('1'), JointCondition.MILD)
        self.assertEqual(JointCondition.parse('2'), JointCondition.MEDIUM)
        self.assertEqual(JointCondition.parse('3'), JointCondition.EXTREAM)
        self.assertEqual(JointCondition.parse('4'), JointCondition.FUSED)
        self.assertEqual(JointCondition.parse('5'), JointCondition.SCHMORALS_NODES)
        self.assertEqual(JointCondition.parse('6'), JointCondition.FRACTURE)

        self.assertEqual(JointCondition.parse(0), JointCondition.NORMAL)

        with self.assertRaises(ValueError):
            JointCondition.parse('')

    def test_avg(self):
        va1 = None
        va2 = JointCondition.MILD
        self.assertEqual(JointCondition.avg(va1, va2), JointCondition.MILD)
        self.assertEqual(JointCondition.avg(va2, va1), JointCondition.MILD)

        va1 = None
        va2 = None
        self.assertEqual(JointCondition.avg(va1, va2), None)
        self.assertEqual(JointCondition.avg(va2, va1), None)

        va1 = JointCondition.MILD
        va2 = JointCondition.EXTREAM
        self.assertEqual(JointCondition.avg(va1, va2), JointCondition.MEDIUM)
        self.assertEqual(JointCondition.avg(va2, va1), JointCondition.MEDIUM)

    def test_order(self):
        self.assertEqual(sorted([JointCondition.FRACTURE, JointCondition.NORMAL]), [JointCondition.NORMAL, JointCondition.FRACTURE])
        # None is sorted first
        self.assertEqual(sorted([JointCondition.FRACTURE, JointCondition.NORMAL, None]), [None, JointCondition.NORMAL, JointCondition.FRACTURE])
        # Numbers are sorted as if they where JointCondition.value
        self.assertEqual(sorted([0, 3, JointCondition.MILD]), [0, JointCondition.MILD, 3])
        # Invalid numbers fail
        with self.assertRaises(ValueError):
            [JointCondition.MILD, -2].sort()


class JointsTest(unittest.TestCase):
    def test_to_pd_data_frame(self):
        shoulder = LeftRight(JointCondition.NORMAL, JointCondition.NORMAL)
        elbow = LeftRight(None, None)
        wrist = LeftRight(JointCondition.NORMAL, None)
        hip = LeftRight(None, JointCondition.NORMAL)
        knee = LeftRight(JointCondition.NORMAL, JointCondition.MEDIUM)
        ankle = LeftRight(None, JointCondition.FRACTURE)

        sacro_illiac = JointCondition.NORMAL
        c1_3 = None
        c4_7 = JointCondition.EXTREAM
        t1_4 = JointCondition.NORMAL
        t5_8 = None
        t9_12 = JointCondition.FRACTURE
        l1_5 = JointCondition.NORMAL

        df = Joints(shoulder, elbow, wrist, hip, knee, ankle, sacro_illiac, c1_3, c4_7, t1_4, t5_8, t9_12, l1_5).to_pd_data_frame('id1')

        with open_binary(bioarch_test, 'JointsTest.test_to_pd_data_frame.json') as json_stream:
            expected_json = json.load(json_stream)

        print(df.to_json(orient='records'))
        actual_json = json.loads(df.to_json(orient='records'))

        self.assertEqual(actual_json, expected_json)

    # https://github.com/TheBiggerGuy/bioarch/issues/21
    def test_to_pd_data_frame_joints_thoracic_max(self):
        shoulder = LeftRight(JointCondition.NORMAL, JointCondition.NORMAL)
        elbow = LeftRight(None, None)
        wrist = LeftRight(JointCondition.NORMAL, None)
        hip = LeftRight(None, JointCondition.NORMAL)
        knee = LeftRight(JointCondition.NORMAL, JointCondition.MEDIUM)
        ankle = LeftRight(None, JointCondition.FRACTURE)

        sacro_illiac = JointCondition.NORMAL
        c1_3 = None
        c4_7 = JointCondition.EXTREAM

        t1_4 = JointCondition.NORMAL
        t5_8 = None
        t9_12 = JointCondition.FRACTURE

        l1_5 = JointCondition.NORMAL

        df = Joints(shoulder, elbow, wrist, hip, knee, ankle, sacro_illiac, c1_3, c4_7, t1_4, t5_8, t9_12, l1_5).to_pd_data_frame('id1')

        thoracic_min = df['thoracic_min']
        thoracic_max = df['thoracic_max']

        self.assertEqual(thoracic_min.to_json(orient='records'), '["NORMAL"]')
        self.assertEqual(thoracic_max.to_json(orient='records'), '["FRACTURE"]')


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


import functools
from typing import Optional


import pandas as pd


from .age import EstimatedAge
from .context import Context
from .joints import Joints
from .left_right import LeftRight
from .mouth import Mouth
from .occupational_markers import OccupationalMarkers
from .sex import Sex
from .trauma import Trauma


# Notes:
#  * https://www.archaeologists.net/sites/default/files/ifa_paper_7.pdf


class BurialInfo(object):
    """docstring for BurialInfo"""
    def __init__(self, site_name: str, site_id: str):
        if site_name is None or site_id is None:
            raise ValueError('site_name and site_id required')
        if site_name == '' or site_id == '':
            raise ValueError('site_name and site_id required')
        self.name = site_name
        self.id = site_id

    def to_pd_series(self, prefix=''):
        labels = [f'{prefix}{label}' for label in ['name', 'id']]
        return pd.Series([self.name, self.id], index=labels, copy=True)


@functools.total_ordering
class LongBoneMeasurement(object):
    """docstring for LongBoneMeasurement"""

    def __init__(self, _max: Optional[float], bi: Optional[float], head: Optional[float], distal: Optional[float]):
        self.max = _max
        self.bi = bi
        self.head = head
        self.distal = distal

    @staticmethod
    def empty():
        return LongBoneMeasurement(None, None, None, None)

    @staticmethod
    def empty_lr():
        return LeftRight(LongBoneMeasurement.empty(), LongBoneMeasurement.empty())

    def to_pd_series(self, prefix=''):
        labels = [f'{prefix}{label}' for label in ['max', 'bi', 'head', 'distal']]
        return pd.Series([self.max, self.bi, self.head, self.distal], index=labels, copy=True)

    def __eq__(self, other):
        if type(other) != type(self):  # pylint: disable=C0123
            raise NotImplementedError
        return ((self.max, self.bi, self.head, self.distal) == (other.max, other.bi, other.head, other.distal))  # pylint: disable=C0325

    def __lt__(self, other):
        if type(other) != type(self):  # pylint: disable=C0123
            raise NotImplementedError
        return ((self.max, self.bi, self.head, self.distal) < (other.max, other.bi, other.head, other.distal))  # pylint: disable=C0325


class OsteologicalSex(object):
    """docstring for OsteologicalSex"""
    def __init__(self, pelvic: Optional[Sex], cranium: Optional[Sex], combined: Optional[Sex]):
        self.pelvic = pelvic
        self.cranium = cranium
        self.combined = combined

    @staticmethod
    def empty():
        return OsteologicalSex(None, None, None)

    def to_pd_data_frame(self, index):
        d = {
            'id': pd.Series([index]),
        }
        for l in ('pelvic', 'cranium', 'combined'):
            val = getattr(self, l)
            d[f'{l}_cat'] = pd.Series([val.name if val else None],  copy=True, dtype=Sex.dtype())  # noqa: E241
            d[f'{l}_val'] = pd.Series([val.value if val else None], copy=True, dtype='Int64')
            val_bin = val.as_bin() if val else None
            d[f'{l}_bin_cat'] = pd.Series([val_bin.name if val_bin else None],  copy=True, dtype=Sex.dtype())  # noqa: E241
            d[f'{l}_bin_val'] = pd.Series([val_bin.value if val_bin else None], copy=True, dtype='Int64')

        return pd.DataFrame.from_dict(d).set_index('id')


class AgeSexStature(object):
    """docstring for AgeSexStature"""

    __slots__ = ['osteological_sex', 'age', 'femur', 'humerus', 'tibia', 'stature', 'body_mass']

    def __init__(self, osteological_sex: OsteologicalSex, age: EstimatedAge, femur: LongBoneMeasurement, humerus: LongBoneMeasurement, tibia: LongBoneMeasurement, stature: Optional[str], body_mass: Optional[str]):
        # Sex
        self.osteological_sex = osteological_sex
        # Age
        self.age = age
        # Long bones
        self.femur = femur
        self.humerus = humerus
        self.tibia = tibia
        # Other
        self.stature = stature
        self.body_mass = body_mass

    @staticmethod
    def empty():
        return AgeSexStature(OsteologicalSex.empty(), EstimatedAge.empty(), LongBoneMeasurement.empty_lr(), LongBoneMeasurement.empty_lr(), LongBoneMeasurement.empty_lr(), None, None)

    def to_pd_data_frame(self, index):
        data = {
            'id': pd.Series([index]),
            'stature': pd.Series([self.stature]),
            'body_mass': pd.Series([self.body_mass]),
        }
        df = pd.DataFrame.from_dict(data).set_index('id')

        s = pd.Series([])
        for bone in ('femur', 'humerus', 'tibia'):
            lr_val = getattr(self, bone)
            s = s.append(lr_val.left.to_pd_series(prefix=f'{bone}_left_'))
            s = s.append(lr_val.right.to_pd_series(prefix=f'{bone}_right_'))
            s = s.append(lr_val.avg().to_pd_series(prefix=f'{bone}_avg_'))
        long_bones = pd.DataFrame.from_dict({index: s}, orient='index')

        age = self.age.to_pd_data_frame(index).add_prefix(f'age_').rename(columns={f'age_id': 'id'})
        oss = self.osteological_sex.to_pd_data_frame(index).add_prefix(f'osteological_sex_').rename(columns={f'osteological_sex_id': 'id'})

        return df.join(long_bones, on='id', how='outer') \
                 .join(age, on='id', how='outer') \
                 .join(oss, on='id', how='outer')


class Individual(object):
    """docstring for Individual"""
    def __init__(self, _id: str, site: BurialInfo, age_sex_stature: AgeSexStature, mouth: Mouth, occupational_markers: OccupationalMarkers, joints: Joints, trauma: Trauma, context: Context):
        self.id = _id
        self.site = site
        self.age_sex_stature = age_sex_stature
        self.mouth = mouth
        self.occupational_markers = occupational_markers
        self.joints = joints
        self.trauma = trauma
        self.context = context

    def to_pd_data_frame(self):
        s = pd.Series([self.id], index=['id'], copy=True)
        s = s.append(self.site.to_pd_series(prefix='site_'))
        s = s.append(self.mouth.to_pd_series(prefix='mouth_'))

        joints_df = self.joints.to_pd_data_frame(self.id).add_prefix('joints_').rename(columns={f'joints_id': 'id'})
        ass_df = self.age_sex_stature.to_pd_data_frame(self.id).add_prefix('ass_').rename(columns={f'ass_id': 'id'})
        om_df = self.occupational_markers.to_pd_data_frame(self.id).add_prefix('om_').rename(columns={f'om_id': 'id'})
        trauma_df = self.trauma.to_pd_data_frame(self.id).add_prefix('trauma_').rename(columns={f'trauma_id': 'id'})
        context_df = self.context.to_pd_data_frame(self.id).add_prefix('context_').rename(columns={f'context_id': 'id'})

        df = pd.DataFrame.from_dict({self.id: s}, orient='index')
        return df.join(joints_df, on='id', how='outer') \
                 .join(ass_df, on='id', how='outer') \
                 .join(om_df, on='id', how='outer') \
                 .join(trauma_df, on='id', how='outer') \
                 .join(context_df, on='id', how='outer')


if __name__ == "__main__":
    raise RuntimeError('No main available')



#!/usr/bin/env python


import logging
from typing import List, Optional, Union


from ensure import check, ensure_annotations
import pandas as pd


logger = logging.getLogger(__name__)


VALID_TEETH    = ('NA', '0', '1', 'A', 'B1', 'B2', 'C', 'D', 'E', 'F', 'G', 'H', 'I')  # noqa: E221
VALID_CALCULUS = ('NA', '0', '1', '2', '3')
VALID_EH       = ('NA', '0', '1')  # noqa: E221
VALID_CAVITIES = ('NA', '0', '1')
VALID_ABCESS   = ('NA', '0', '1')  # noqa: E221


class Tooth(object):
    """docstring for Tooth"""

    __slots__ = ['_tooth', '_calculus', '_eh', '_cavities', '_abcess']

    @ensure_annotations
    def __init__(self, tooth: str, calculus: str, eh: str, cavities: str, abcess: str):
        check(tooth).is_in(VALID_TEETH).or_raise(ValueError)
        check(calculus).is_in(VALID_CALCULUS).or_raise(ValueError)
        check(eh).is_in(VALID_EH).or_raise(ValueError)
        check(cavities).is_in(VALID_CAVITIES).or_raise(ValueError)
        check(abcess).is_in(VALID_ABCESS).or_raise(ValueError)
        # If there is no tooth then there can be no calculus, eh or cavities.
        # Note there can be abcess
        if tooth == 'NA':
            check(calculus).equals('NA').or_raise(ValueError)
            check(eh).equals('NA').or_raise(ValueError)
            check(cavities).equals('NA').or_raise(ValueError)

        self._tooth: str = tooth
        self._calculus: str = calculus
        self._eh: str = eh
        self._cavities: str = cavities
        self._abcess: str = abcess

    @property
    def tooth(self) -> str:
        """
        'NA', '0', '1', 'A', 'B1', 'B2', 'C', 'D', 'E', 'F', 'G', 'H', 'I'
        """
        return self._tooth

    @property
    def calculus(self) -> str:
        """
        'NA', '0', '1', '2', '3'
        """
        return self._calculus

    @property
    def eh(self) -> str:
        """
        'NA', '0', '1'
        """
        return self._eh

    @property
    def cavities(self) -> str:
        """
        'NA', '0', '1'
        """
        return self._cavities

    @property
    def abcess(self) -> str:
        """
        'NA', '0', '1'
        """
        return self._abcess

    @staticmethod
    def empty():
        return Tooth('NA', 'NA', 'NA', 'NA', 'NA')

    def _to_pd_value(self, label: str) -> Optional[Union[int, bool]]:
        val = getattr(self, label)
        if val == 'NA':
            return None

        if label == 'tooth':
            return VALID_TEETH.index(val) - 1
        if label == 'calculus':
            return VALID_CALCULUS.index(val) - 1
        if label in ('eh', 'cavities', 'abcess'):
            return bool(int(val))
        raise RuntimeError

    def to_pd_series(self, prefix=''):
        labels = []
        values = []
        for label in self.__slots__:
            label = label[1:]
            labels.append(f'{prefix}{label}')
            val = getattr(self, label)
            values.append(val)
            labels.append(f'{prefix}{label}_val')
            values.append(self._to_pd_value(label))
        return pd.Series(values, index=labels, copy=True)

    def __eq__(self, other):
        if other is None:
            return False
        if type(other) != type(self):  # pylint: disable=C0123
            raise NotImplementedError
        return (self.tooth, self.calculus, self.eh, self.cavities, self.abcess) == (other.tooth, other.calculus, other.eh, other.cavities, other.abcess)


TOOTH_GROUPS = {
    'all':       [x - 1 for x in list(range(1, 33))],    # noqa: E241
    'molar':     [x - 1 for x in [1, 2, 3, 14, 15, 16, 17, 18, 19, 30, 31, 32]],    # noqa: E241
    'premolars': [x - 1 for x in [4, 5, 12, 13, 20, 21, 28, 29]],
    'canines':   [x - 1 for x in [6, 11, 22, 27]],    # noqa: E241
    'incisors':  [x - 1 for x in [7, 8, 9, 10, 23, 24, 25, 26]],    # noqa: E241
}


class Mouth(object):
    """Object to hold the 32 teeth"""
    def __init__(self, teeth: List[Tooth]):
        if len(teeth) != 32:
            raise ValueError(f'Incorrect number of teeth: {len(teeth)}')
        self.teeth = teeth

    @staticmethod
    def empty():
        return Mouth([Tooth.empty()] * 32)

    def _to_pd_series_group(self, group, prefix, include_all=False):
        prefix = f'{prefix}{group}_'
        teeth = [tooth for i, tooth in enumerate(self.teeth) if i in TOOTH_GROUPS[group]]

        per_tooth = pd.Series([], index=[])
        for i, tooth in enumerate(teeth):
            per_tooth = per_tooth.append(tooth.to_pd_series(prefix=f'{prefix}tooth_{i}_'))

        number_of_teeth = sum([1 for t in teeth if t.tooth != 'NA'])
        summaray = pd.Series([number_of_teeth], index=[f'{prefix}number_of_teeth'], copy=True)
        for label in Tooth.__slots__:
            label = label[1:]
            subset = pd.Series([v for k, v in per_tooth.items() if label in k and k.endswith('_val')])
            summaray = summaray.append(pd.Series([subset.mean(skipna=True)], index=[f'{prefix}{label}_mean']))
            summaray = summaray.append(pd.Series([subset.max(skipna=True)], index=[f'{prefix}{label}_max']))
            summaray = summaray.append(pd.Series([subset.min(skipna=True)], index=[f'{prefix}{label}_min']))
            summaray = summaray.append(pd.Series([subset.count()], index=[f'{prefix}{label}_count']))

        result = summaray
        if include_all:
            result = per_tooth.append(result)
        return result

    def to_pd_series(self, prefix=''):
        return pd.concat([self._to_pd_series_group(group, prefix, include_all=(group == 'all')) for group in TOOTH_GROUPS])


if __name__ == "__main__":
    raise RuntimeError('No main available')



#!/usr/bin/env python


import unittest


from .age import AgeCategory, EstimatedAge


class AgeCategoryTest(unittest.TestCase):
    def test_order(self):
        self.assertEqual(sorted([AgeCategory.OLD, AgeCategory.YOUNG]), [AgeCategory.YOUNG, AgeCategory.OLD])
        # None is sorted first
        self.assertEqual(sorted([AgeCategory.OLD, AgeCategory.YOUNG, None]), [None, AgeCategory.YOUNG, AgeCategory.OLD])
        # Numbers are sorted as if they where AgeCategory.value
        self.assertEqual(sorted([0, 3, AgeCategory.YOUNG]), [0, AgeCategory.YOUNG, 3])
        # Invalid numbers fail
        with self.assertRaises(ValueError):
            [AgeCategory.OLD, -1].sort()

    def test_parse(self):
        self.assertEqual(AgeCategory.parse('UNKNOWN'), AgeCategory.UNKNOWN)
        self.assertEqual(AgeCategory.parse('YOUNG'), AgeCategory.YOUNG)
        self.assertEqual(AgeCategory.parse('YOUNG_ADULT'), AgeCategory.YOUNG_ADULT)
        self.assertEqual(AgeCategory.parse('YOUNG ADULT'), AgeCategory.YOUNG_ADULT)
        self.assertEqual(AgeCategory.parse('ADULT'), AgeCategory.ADULT)
        self.assertEqual(AgeCategory.parse('MIDDLE'), AgeCategory.MIDDLE)
        self.assertEqual(AgeCategory.parse('MIDDLE_OLD'), AgeCategory.MIDDLE_OLD)
        self.assertEqual(AgeCategory.parse('MIDDLE/OLD'), AgeCategory.MIDDLE_OLD)
        self.assertEqual(AgeCategory.parse('OLD'), AgeCategory.OLD)
        self.assertEqual(AgeCategory.parse('OA'), AgeCategory.OLD)

        self.assertEqual(AgeCategory.parse('UNKNOWN'), AgeCategory.parse('unknown'))
        self.assertEqual(AgeCategory.parse(None), None)
        self.assertEqual(AgeCategory.parse(AgeCategory.UNKNOWN), AgeCategory.UNKNOWN)

    def test_to_quad(self):
        self.assertEqual(AgeCategory.UNKNOWN.as_quad(), AgeCategory.UNKNOWN)
        self.assertEqual(AgeCategory.YOUNG.as_quad(), AgeCategory.YOUNG)
        self.assertEqual(AgeCategory.YOUNG_ADULT.as_quad(), AgeCategory.YOUNG)
        self.assertEqual(AgeCategory.ADULT.as_quad(), AgeCategory.ADULT)
        self.assertEqual(AgeCategory.MIDDLE.as_quad(), AgeCategory.MIDDLE)
        self.assertEqual(AgeCategory.MIDDLE_OLD.as_quad(), AgeCategory.MIDDLE)
        self.assertEqual(AgeCategory.OLD.as_quad(), AgeCategory.OLD)


class EstimatedAgeTest(unittest.TestCase):
    def test_costructor(self):
        self.assertEqual(EstimatedAge('UNKNOWN', 'UNKNOWN').category, AgeCategory.UNKNOWN)
        self.assertEqual(EstimatedAge('OLD', 'UNKNOWN').category, AgeCategory.OLD)

        with self.assertRaises(ValueError):
            EstimatedAge('NA_AN_AGE', 'UNKNOWN')

        self.assertEqual(EstimatedAge('OLD', '10-20').ranged, range(10, 20))
        self.assertEqual(EstimatedAge('OLD', '20+').ranged, range(20, 100))
        self.assertEqual(EstimatedAge('OLD', '=20').ranged, range(20, 21))
        self.assertEqual(EstimatedAge('OLD', 'UNKNOWN').ranged, None)
        self.assertEqual(EstimatedAge('OLD', '?').ranged, None)
        self.assertEqual(EstimatedAge('OLD', None).ranged, None)

    def test_to_pd_data_frame(self):
        df = EstimatedAge('UNKNOWN', 'UNKNOWN').to_pd_data_frame('id1')
        self.assertEqual(df.to_json(orient='records'), '[{"category_cat":"UNKNOWN","category_val":0,"category_quad_cat":"UNKNOWN","category_quad_val":0}]')

        df = EstimatedAge('OLD', '45-60').to_pd_data_frame('id1')
        self.assertEqual(df.to_json(orient='records'), '[{"category_cat":"OLD","category_val":5,"category_quad_cat":"OLD","category_quad_val":5,"ranged":[45,46,47,48,49,50,51,52,53,54,55,56,57,58,59]}]')


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


import enum
from enum import Enum
import functools
import logging
from statistics import mean


import pandas as pd
from pandas.api.types import CategoricalDtype


from .left_right import LeftRight


logger = logging.getLogger(__name__)


@functools.total_ordering
@enum.unique
class JointCondition(Enum):
    """
    1 - slight/mild
    2 - medium
    3 - extreme
    4 - fused
    5 - SCHMORALS NODES
    6 - FRACTURE
    """
    NORMAL          =  0  # noqa: E221,E222
    MILD            =  1  # noqa: E221,E222
    MEDIUM          =  2  # noqa: E221,E222
    EXTREAM         =  3  # noqa: E221,E222
    FUSED           =  4  # noqa: E221,E222
    SCHMORALS_NODES =  5  # noqa: E221,E222
    FRACTURE        =  6  # noqa: E221,E222

    @staticmethod
    def parse(value):
        if value is None:
            return None
        if type(value) == JointCondition:  # pylint: disable=C0123
            return value
        if isinstance(value, int):
            value = str(value)
        if not isinstance(value, str):
            raise ValueError(f'Failed to parse JointCondition: "{value}"')
        value = value.upper()
        for condition in JointCondition:
            if value == condition.name:
                return condition
            if value == str(condition.value):
                return condition
        if value in ('NA', 'N'):
            return None
        logger.error('Failed to parse JointCondition: "%s"', value)
        raise ValueError

    @staticmethod
    def avg(left, right):
        if left is None:
            return right
        if right is None:
            return left
        return JointCondition(int(mean((left.value, right.value))))

    def __lt__(self, other):
        if other is None:
            return False
        if isinstance(other, int):
            other = JointCondition(other)
        if type(other) != type(self):  # pylint: disable=C0123
            logger.warning('Attempt to compare: %s with %s', self, other)
            raise NotImplementedError
        return (self.value < other.value)  # pylint: disable=C0325,W0143

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        return self.name

    @staticmethod
    def dtype():
        return CategoricalDtype(categories=[s.name for s in JointCondition], ordered=True)


JOINTS_SUMMARY_STATS = {
    'cervical': set(['c1_3', 'c4_7']),
    'thoracic': set(['t1_4', 't5_8', 't9_12']),
    'lumbar': set(['l1_5']),
}


class Joints(object):  # pylint: disable=R0902
    """docstring for Joints"""
    def __init__(self, shoulder: LeftRight[JointCondition], elbow: LeftRight[JointCondition], wrist: LeftRight[JointCondition], hip: LeftRight[JointCondition], knee: LeftRight[JointCondition], ankle: LeftRight[JointCondition], sacro_illiac: JointCondition, c1_3: JointCondition, c4_7: JointCondition, t1_4: JointCondition, t5_8: JointCondition, t9_12: JointCondition, l1_5: JointCondition):
        self.shoulder = shoulder
        self.elbow = elbow
        self.wrist = wrist
        self.hip = hip
        self.knee = knee
        self.ankle = ankle
        self.sacro_illiac = sacro_illiac
        self.c1_3 = c1_3
        self.c4_7 = c4_7
        self.t1_4 = t1_4
        self.t5_8 = t5_8
        self.t9_12 = t9_12
        self.l1_5 = l1_5

    @staticmethod
    def empty():
        args = [LeftRight(None, None)] * 6
        args += [None] * 7
        return Joints(*args)

    def to_pd_data_frame(self, index):
        data = {
            'id': pd.Series([index]),
        }
        for key, value in self.__dict__.items():
            if isinstance(value, LeftRight):
                data[f'{key}_left'] = pd.Series([value.left.name if value.left else None], copy=True, dtype=JointCondition.dtype())
                data[f'{key}_right'] = pd.Series([value.right.name if value.right else None], copy=True, dtype=JointCondition.dtype())
                avg = value.avg()
                data[f'{key}_avg'] = pd.Series([avg.name if avg else None], copy=True, dtype=JointCondition.dtype())
            else:
                data[f'{key}'] = pd.Series([value.name if value else None], copy=True, dtype=JointCondition.dtype())

        for prefix, cols in JOINTS_SUMMARY_STATS.items():
            subset = [value for key, value in self.__dict__.items() if key in cols and value is not None]
            min_val = min(subset) if subset else None
            max_val = max(subset) if subset else None
            count = len(subset)
            data[f'{prefix}_min'] = pd.Series([min_val.name if min_val else None], copy=True, dtype=JointCondition.dtype())
            data[f'{prefix}_max'] = pd.Series([max_val.name if max_val else None], copy=True, dtype=JointCondition.dtype())
            data[f'{prefix}_count'] = pd.Series([count], copy=True)

        return pd.DataFrame.from_dict(data).set_index('id')


if __name__ == "__main__":
    raise RuntimeError('No main available')



#!/usr/bin/env python


import enum
from enum import Enum
import functools
import logging
from typing import Any, cast, Dict, Optional


import pandas as pd
from pandas.api.types import CategoricalDtype


logger = logging.getLogger(__name__)


@functools.total_ordering
@enum.unique
class CompassBearing(Enum):
    NORTH      = 0  # noqa: E221
    NORTH_EAST = 1
    EAST       = 2  # noqa: E221
    SOUTH_EAST = 3
    SOUTH      = 4  # noqa: E221
    SOUTH_WEST = 5
    WEST       = 6  # noqa: E221
    NORTH_WEST = 7

    @staticmethod
    def parse(value: Any) -> Optional['CompassBearing']:
        if value is None:
            return None
        if type(value) == CompassBearing:  # pylint: disable=C0123
            return cast(CompassBearing, value)
        if not isinstance(value, str):
            raise ValueError(f'Failed to parse {CompassBearing.__name__}: "{value}"')
        value = value.upper()
        for bearing in CompassBearing:
            if value == bearing.name:
                return bearing
            if value == bearing.to_short_code():
                return bearing
        raise ValueError(f'Failed to parse {CompassBearing.__name__}: "{value}"')

    def to_short_code(self):
        return ''.join([p[0] for p in str(self.name).split('_')])

    def __lt__(self, other):
        if other is None:
            return False
        if isinstance(other, int):
            other = CompassBearing(other)
        if type(other) != type(self):  # pylint: disable=C0123
            logger.warning('Attempt to compare: %s with %s', self, other)
            raise NotImplementedError
        return (self.value < other.value)  # pylint: disable=C0325,W0143

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        return self.name

    @staticmethod
    def dtype():
        return CategoricalDtype(categories=[s.name for s in CompassBearing], ordered=True)


@functools.total_ordering
@enum.unique
class Present(Enum):
    NOT_PRESENT = 0
    PRESENT     = 1  # noqa: E221

    @staticmethod
    def parse(value: Any) -> Optional['Present']:
        if value is None:
            return None
        if isinstance(value, str) and value.upper() == 'NA':
            return None
        if type(value) == Present:  # pylint: disable=C0123
            return cast(Present, value)
        if isinstance(value, bool):
            return Present.PRESENT if value else Present.NOT_PRESENT
        value = float(value)
        if value == 0.0:
            return Present.NOT_PRESENT
        if value > 0.0:
            return Present.PRESENT
        raise ValueError(f'Failed to parse {Present.__name__}: "{value}"')

    def __lt__(self, other):
        if other is None:
            return False
        if isinstance(other, int):
            other = Present(other)
        if type(other) != type(self):  # pylint: disable=C0123
            logger.warning('Attempt to compare: %s with %s', self, other)
            raise NotImplementedError
        return (self.value < other.value)  # pylint: disable=C0325,W0143

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        return self.name

    @staticmethod
    def dtype():
        return CategoricalDtype(categories=[s.name for s in Present], ordered=True)


@functools.total_ordering
@enum.unique
class BodyPosition(Enum):
    SUPINE              = 0  # noqa: E221
    SUPINE_FIXED_LEGS   = 1  # noqa: E221
    CROUCHED            = 2  # noqa: E221
    CROUCHED_LEFT_SIDE  = 3  # noqa: E221
    CROUCHED_RIGHT_SIDE = 4
    STOMACH             = 5  # noqa: E221

    @staticmethod
    def parse(value: Any) -> Optional['BodyPosition']:
        if value is None:
            return None
        if isinstance(value, str) and value.upper() == 'NA':
            return None
        if type(value) == BodyPosition:  # pylint: disable=C0123
            return cast(BodyPosition, value)
        if isinstance(value, str):
            value = value.upper()
        for position in BodyPosition:
            if value == position.name:
                return position
            if value == position.value:
                return position
        raise ValueError(f'Failed to parse {BodyPosition.__name__}: "{value}"')

    def __lt__(self, other):
        if other is None:
            return False
        if isinstance(other, int):
            other = BodyPosition(other)
        if type(other) != type(self):  # pylint: disable=C0123
            logger.warning('Attempt to compare: %s with %s', self, other)
            raise NotImplementedError
        return (self.value < other.value)  # pylint: disable=C0325,W0143

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        return self.name

    @staticmethod
    def dtype():
        return CategoricalDtype(categories=[s.name for s in BodyPosition], ordered=True)


KNOWN_GROUPS = {
    'utilitarian'  : set(['knife', 'whetstone', 'awl', 'scissors', 'vessel', 'pot_sherd', 'flint', 'flakes', 'flint_flakes']),  # noqa: E203
    'textile'      : set(['textile', 'needle', 'spindle_whorl']),  # noqa: E203
    'equestrian'   : set(['equestrian', 'horse_equipment']),  # noqa: E203
    'economic'     : set(['coins', 'scales']),  # noqa: E203
    'organic_material': set(['animal_remains', 'charcoal', 'flower', 'shell', 'burned_bones']),
    'appearance'   : set(['brooch', 'cloak ring', 'bracelet', 'beads', 'comb', 'buckle', 'mounts', 'cloak_fastener', 'iron_ring']),  # noqa: E203
    'burial_container': set(['coffin', 'coffin_substitute']),
    'weapons'      : set(['sword', 'axe', 'shield_boss', 'spear']),  # noqa: E203
    'iron_fragment': set(['iron frafments', 'nails', 'iron_object', 'iron_nail/rivets']),
    'miscellaneous': set(['lock', 'keys', 'thors_hammer', 'bronze_item', 'bronze_disk', 'quartz', 'unidentified_bronze', 'iron_pole']),
}


class Context(object):
    """docstring for Context"""

    def __init__(self, body_position: Optional[BodyPosition], body_orientation: Optional[CompassBearing], disturbed: Optional[Present], decapitation: Optional[Present], double_grave: Optional[Present], stone_layer: Optional[Present], grave_goods: Dict[str, Optional[Any]]):
        if body_position is not None and not isinstance(body_position, BodyPosition):
            raise ValueError(f'Invalid body_position: "{body_position}"')
        self.body_position = body_position

        if body_orientation is not None and not isinstance(body_orientation, CompassBearing):
            raise ValueError(f'Invalid body_orientation: "{body_orientation}"')
        self.body_orientation = body_orientation

        self.disturbed = disturbed
        self.decapitation = decapitation
        self.double_grave = double_grave
        self.stone_layer = stone_layer

        self.grave_goods = {k.lower(): Present.parse(v) for k, v in grave_goods.items()}

        countable_goods = [float(v) for v in grave_goods.values() if Present.parse(v) is not None]  # type: ignore
        self.grave_goods_total = sum(countable_goods) if len(countable_goods) > 0 else None

    @staticmethod
    def empty():
        return Context(None, None, None, None, None, None, {})

    @staticmethod
    def group(value):
        value = value.lower()
        groups = set()
        for group_name, group in KNOWN_GROUPS.items():
            if value in group:
                groups.add(group_name)
        return groups

    def to_pd_data_frame(self, index):
        data = {
            'id': pd.Series([index]),
        }

        data['body_position_cat'] = pd.Series([self.body_position.name if self.body_position else None], copy=True, dtype=BodyPosition.dtype())
        data['body_position_val'] = pd.Series([self.body_position.value if self.body_position else None], copy=True, dtype='Int64')

        data['body_orientation_cat'] = pd.Series([self.body_orientation.name if self.body_orientation else None], copy=True, dtype=CompassBearing.dtype())
        data['body_orientation_val'] = pd.Series([self.body_orientation.value if self.body_orientation else None], copy=True, dtype='Int64')

        data['disturbed_cat'] = pd.Series([self.disturbed.name if self.disturbed else None], copy=True, dtype=Present.dtype())
        data['decapitation_cat'] = pd.Series([self.decapitation.name if self.decapitation else None], copy=True, dtype=Present.dtype())
        data['double_grave_cat'] = pd.Series([self.double_grave.name if self.double_grave else None], copy=True, dtype=Present.dtype())
        data['stone_layer_cat'] = pd.Series([self.stone_layer.name if self.stone_layer else None], copy=True, dtype=Present.dtype())

        for key, value in self.grave_goods.items():
            data[f'all_{key}_cat'] = pd.Series([value.name if value else None], copy=True, dtype=Present.dtype())
            data[f'all_{key}_val'] = pd.Series([value.value if value else None], copy=True, dtype='Int64')

        per_group_count = None
        for group_name, group in KNOWN_GROUPS.items():
            status_set = {v for k, v in self.grave_goods.items() if k in group and v}
            if Present.PRESENT in status_set:
                present = Present.PRESENT
                per_group_count = per_group_count + 1 if per_group_count else 1
            elif Present.NOT_PRESENT in status_set:
                present = Present.NOT_PRESENT
                per_group_count = per_group_count + 0 if per_group_count else 0
            else:
                present = None
            data[f'{group_name}_cat'] = pd.Series([present.name if present else None], copy=True, dtype=Present.dtype())
            data[f'{group_name}_val'] = pd.Series([present.value if present else None], copy=True, dtype='Int64')

        data['total_grave_goods'] = pd.Series([self.grave_goods_total], copy=True, dtype='Int64')
        data['total_grave_goods_indicator'] = pd.Series([per_group_count], copy=True, dtype='Int64')

        return pd.DataFrame.from_dict(data).set_index('id')


if __name__ == "__main__":
    raise RuntimeError('No main available')



#!/usr/bin/env python3

from os import getenv, path
import re
import sys

# Always prefer setuptools over distutils
from setuptools import find_packages, setup
from setuptools.command.install import install
from setuptools.command.test import test as TestCommand  # noqa: N812


class MetatDataFetcher(object):
    """docstring for MetatDataFetcher"""
    def __init__(self, module_name):
        pwd = path.abspath(path.dirname(__file__))

        with open(path.join(pwd, module_name, '__init__.py'), mode='r', encoding='utf-8') as fd:
            contents = fd.read()
        version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                            contents, re.MULTILINE).group(1)
        title = re.search(r'^__title__\s*=\s*[\'"]([^\'"]*)[\'"]',
                          contents, re.MULTILINE).group(1)
        author = re.search(r'^__author__\s*=\s*[\'"]([^\'"]*)[\'"]',
                           contents, re.MULTILINE).group(1)
        if not version:
            raise RuntimeError('Cannot find version information')
        if not title:
            raise RuntimeError('Cannot find title information')
        if not author:
            raise RuntimeError('Cannot find author information')
        self.version = version
        self.title = title
        self.author = author

        # Get the long description from the README file
        with open(path.join(pwd, 'README.md'), mode='r', encoding='utf-8') as f:
            long_description = f.read()
        if not long_description:
            raise RuntimeError('Cannot find long_description information')
        self.long_description = long_description


meta_data = MetatDataFetcher('bioarch')


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        circleci_tag = getenv('CIRCLE_TAG')

        if circleci_tag is None:
            sys.exit(f'Faied to find "CIRCLE_TAG" env')

        if circleci_tag != ('v' + meta_data.version):
            info = f'CircleCI Git tag "{circleci_tag}" does not match the version of this app "{meta_data.version}"'
            sys.exit(info)


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ''  # pylint: disable=W0201

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import shlex  # pylint: disable=C0415
        import pytest  # pylint: disable=C0415
        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


setup(
    name=meta_data.title,
    version=meta_data.version,
    description='Python bioarchaeology',
    long_description=meta_data.long_description,
    long_description_content_type='text/markdown',
    author=meta_data.author,
    author_email='thebigguy.co.uk@gmail.com',
    url='https://github.com/TheBiggerGuy/bioarch',
    packages=find_packages(exclude=['docs', 'tests']),
    package_data={'': ['LICENSE', 'README.md']},
    zip_safe=False,
    install_requires=[],
    tests_require=['pytest'],
    cmdclass={
        'test': PyTest,
        'verify': VerifyVersionCommand,
    },
    keywords=[],
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        "Programming Language :: Python :: 3 :: Only",
        'Programming Language :: Python :: Implementation :: CPython',
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.7',
)
